#include"SDK/apiPlugin.h"

#include "SDK/apiInternet.h"
#include "SDK/apiCore.h"
#include "SDK/apiActions.h"
#include "SDK/apiMenu.h"
#include "SDK/apiFileManager.h"

#include <windows.h>
#include <wininet.h>

static const wchar_t *const OUTPUT_FILENAME = L"D:/test.html";

IAIMPString *createString(IAIMPCore *core, const wchar_t *content) {
	IAIMPString *str;
	core->CreateObject(IID_IAIMPString, reinterpret_cast<void **>(&str));
	str->SetData(const_cast<WCHAR *>(content), wcslen(content));
	return str;
}

class CookieTestHandler : public IAIMPActionEvent {
public:
	explicit CookieTestHandler(IAIMPCore *core) : core(core) {}

	void WINAPI OnExecute(IUnknown *) override {// Main code here
		IAIMPServiceHTTPClient *client;
		core->QueryInterface(IID_IAIMPServiceHTTPClient, reinterpret_cast<void **>(&client));
		IAIMPServiceFileStreaming *fileService;
		core->QueryInterface(IID_IAIMPServiceFileStreaming, reinterpret_cast<void **>(&fileService));
		IAIMPStream *stream;
		auto fileName = createString(core, OUTPUT_FILENAME);
		fileService->CreateStreamForFile(fileName, AIMP_SERVICE_FILESTREAMING_FLAG_CREATENEW| AIMP_SERVICE_FILESTREAMING_FLAG_READWRITE, -1, -1, &stream);
		fileName->Release();
		fileService->Release();
		auto url = createString(core,L"https://yandex.ru/\r\nCookie:Session_id=");// Insert your session id
		void *task;
		//InternetSetCookie("https://yandex.ru", nullptr,"Session_id=");// Insert your session id. Does not send cookie without it
		client->Get(url, 0, stream, nullptr, nullptr, &task);
		stream->Release();
		url->Release();
		client->Release();
	}

	virtual ~CookieTestHandler() noexcept {
		core->Release();
	};

	HRESULT WINAPI QueryInterface(REFIID riid, LPVOID *ppvObj) override {
		if (!ppvObj) return E_POINTER;
		if (riid == IID_IAIMPActionEvent) {
			*ppvObj = this;
			++refCount;
			return S_OK;
		}
		return E_NOINTERFACE;
	}

	ULONG WINAPI AddRef() override { return ++refCount; }

	ULONG WINAPI Release() override {
		--refCount;
		if (refCount == 0)delete this;
		return refCount;
	}

private:
	ULONG refCount = 1;
	IAIMPCore *core;
};

class CookieTest : public IAIMPPlugin {
public:
	PWCHAR WINAPI InfoGet(int Index) override {
		switch (Index) {
			case AIMP_PLUGIN_INFO_NAME: return const_cast<PWCHAR>(L"Cookie Test");
			case AIMP_PLUGIN_INFO_AUTHOR: return const_cast<PWCHAR>(L"Vedblade");
			case AIMP_PLUGIN_INFO_SHORT_DESCRIPTION: return const_cast<PWCHAR>(L"");
			default: return nullptr;
		}
	}

	DWORD WINAPI InfoGetCategories() override {
		return AIMP_PLUGIN_CATEGORY_ADDONS;
	}

	void WINAPI SystemNotification(int NotifyID, IUnknown *Data) override {}

	HRESULT WINAPI Initialize(IAIMPCore *Core) override {
		IAIMPServiceMenuManager *menuManager;
		auto res = Core->QueryInterface(IID_IAIMPServiceMenuManager, reinterpret_cast<void **>(&menuManager));
		if (FAILED(res))return E_FAIL;
		IAIMPAction *action;
		Core->CreateObject(IID_IAIMPAction, reinterpret_cast<void **>(&action));
		auto actionID = createString(Core, L"CookieTestAction");
		action->SetValueAsObject(AIMP_ACTION_PROPID_ID, actionID);
		actionID->Release();
		Core->AddRef();
		auto handler = new CookieTestHandler(Core);
		action->SetValueAsObject(AIMP_ACTION_PROPID_EVENT, handler);
		handler->Release();
		Core->RegisterExtension(IID_IAIMPServiceActionManager, action);
		IAIMPMenuItem *parent;
		menuManager->GetBuiltIn(AIMP_MENUID_COMMON_UTILITIES, &parent);
		menuManager->Release();
		IAIMPMenuItem *newItem;
		Core->CreateObject(IID_IAIMPMenuItem, reinterpret_cast<void **>(&newItem));
		newItem->SetValueAsObject(AIMP_MENUITEM_PROPID_PARENT, parent);
		parent->Release();
		auto itemID = createString(Core, L"CookieTestMenu");
		newItem->SetValueAsObject(AIMP_MENUITEM_PROPID_ID, itemID);
		itemID->Release();
		newItem->SetValueAsObject(AIMP_MENUITEM_PROPID_ACTION, action);
		action->Release();
		Core->RegisterExtension(IID_IAIMPServiceMenuManager, newItem);
		newItem->Release();
		return S_OK;
	}

	HRESULT WINAPI Finalize() override {
		return S_OK;
	}

	virtual ~CookieTest() noexcept = default;

	HRESULT WINAPI QueryInterface(REFIID riid, LPVOID *ppvObj) override {
		if (!ppvObj) return E_POINTER;
		if (riid == IID_IUnknown) {
			*ppvObj = this;
			++refCount;
			return S_OK;
		}
		return E_NOINTERFACE;
	}

	ULONG WINAPI AddRef() override { return ++refCount; }

	ULONG WINAPI Release() override {
		--refCount;
		if (refCount == 0)delete this;
		return refCount;
	}

private:
	ULONG refCount = 1;
};

extern "C" HRESULT __declspec(dllexport) WINAPI AIMPPluginGetHeader(IAIMPPlugin **Header) {
	*Header = new CookieTest;
	return S_OK;
}

BOOL APIENTRY DllMain(HMODULE, DWORD, LPVOID) { return TRUE; }